package com.qmth.themis.backend.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.backend.util.ServletUtil;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBOrg;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.service.TBOrgService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 机构 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "机构Controller")
@RestController
@RequestMapping("/${prefix.url.admin}/org")
public class TBOrgController {

    @Resource
    TBOrgService tbOrgService;

//    @Resource
//    private CacheManager cacheManager;

    @Resource
    RedisUtil redisUtil;

    //    @CachePut(value = "org_cache", key = "'orgCacheQuery'")
//    @Cacheable(value = "org_cache", key = "#p0")
    @ApiOperation(value = "机构查询接口")
    @RequestMapping(value = "/query", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "机构信息", response = TBOrg.class)})
    public Result query(@ApiParam(value = "机构id", required = false) @RequestParam(required = false) Long id, @ApiParam(value = "机构编码", required = false) @RequestParam(required = false) String code, @ApiParam(value = "机构名称", required = false) @RequestParam(required = false) String name, @ApiParam(value = "是否启用", required = false) @RequestParam(required = false) Integer enable) {
        QueryWrapper<TBOrg> tbOrgQueryWrapper = new QueryWrapper<>();
        if (Objects.nonNull(id)) {
            tbOrgQueryWrapper.lambda().eq(TBOrg::getId, id);
        }
        if (Objects.nonNull(code)) {
            tbOrgQueryWrapper.lambda().eq(TBOrg::getCode, code);
        }
        if (Objects.nonNull(name)) {
            tbOrgQueryWrapper.lambda().like(TBOrg::getName, name);
        }
        if (Objects.nonNull(enable)) {
            tbOrgQueryWrapper.lambda().eq(TBOrg::getEnable, enable);
        }
        List<TBOrg> tbOrgList = tbOrgService.list(tbOrgQueryWrapper);
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, tbOrgList);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "机构查询分页接口")
    @RequestMapping(value = "/queryByPage", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "机构信息", response = TBOrg.class)})
    public Result queryByPage(@ApiParam(value = "机构代码", required = false) @RequestParam(required = false) String code, @ApiParam(value = "机构名称", required = false) @RequestParam(required = false) String name, @ApiParam(value = "是否启用", required = false) @RequestParam(required = false) Integer enable) {
        QueryWrapper<TBOrg> tbOrgQueryWrapper = new QueryWrapper<>();
        if (Objects.nonNull(code)) {
            tbOrgQueryWrapper.lambda().eq(TBOrg::getCode, code);
        }
        if (Objects.nonNull(name)) {
            tbOrgQueryWrapper.lambda().like(TBOrg::getName, name);
        }
        if (Objects.nonNull(enable)) {
            tbOrgQueryWrapper.lambda().eq(TBOrg::getEnable, enable);
        }
        List<TBOrg> tbOrgList = tbOrgService.list(tbOrgQueryWrapper);
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, tbOrgList);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "根据机构代码查询机构信息接口")
    @RequestMapping(value = "/queryByOrgCode", method = RequestMethod.GET)
    @ApiResponses({@ApiResponse(code = 200, message = "机构信息", response = TBOrg.class)})
    public Result queryByOrgCode(@ApiParam(value = "机构code", required = true) @RequestParam(required = false) String code) {
        if (Objects.nonNull(code) || Objects.equals(code, "")) {
            throw new BusinessException(ExceptionResultEnum.ORG_CODE_IS_NULL);
        }
        QueryWrapper<TBOrg> tbOrgQueryWrapper = new QueryWrapper<>();
        tbOrgQueryWrapper.lambda().eq(TBOrg::getCode, code);
        TBOrg tbOrg = tbOrgService.getOne(tbOrgQueryWrapper);
        if (Objects.isNull(tbOrg)) {
            throw new BusinessException(ExceptionResultEnum.ORG_NO);
        }
        Map map = new HashMap<>();
        map.put("logo", tbOrg.getLogo());
        return ResultUtil.ok(map);
    }

    //    @CacheEvict(value = "org_cache", key = "'orgCacheQuery'")
    @ApiOperation(value = "机构新增/编辑接口")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @Transactional
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    public Result save(@ApiParam(value = "机构信息", required = true) @RequestBody TBOrg tbOrg) {
        if (Objects.isNull(tbOrg)) {
            throw new BusinessException(ExceptionResultEnum.ORG_INFO_IS_NULL);
        }
        TBUser tbUser = (TBUser) ServletUtil.getRequestAccount();
        if (Objects.isNull(tbOrg.getId())) {
            tbOrg.setId(Constants.idGen.next());
            tbOrg.setCreateId(tbUser.getId());
        } else {
            if (Objects.nonNull(redisUtil.getOrg(tbOrg.getId()))) {
                redisUtil.setOrg(tbOrg.getId(), tbOrg);
            }
            tbOrg.setUpdateId(tbUser.getId());
        }
        tbOrgService.saveOrUpdate(tbOrg);
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }
}