package com.qmth.themis.backend.start;

import com.qmth.themis.business.config.SystemConfig;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.mq.enums.MqGroupEnum;
import com.qmth.themis.mq.enums.MqTagEnum;
import com.qmth.themis.mq.enums.MqTopicEnum;
import com.qmth.themis.mq.listener.RocketMessageConsumer;
import com.qmth.themis.mq.templete.impl.SessionConcurrentlyImpl;
import com.qmth.themis.mq.templete.impl.TaskConcurrentlyImpl;
import com.qmth.themis.mq.templete.impl.UserLogConcurrentlyImpl;
import com.qmth.themis.mq.templete.impl.WebsocketUnNormalConcurrentlyImpl;
import org.apache.rocketmq.common.protocol.heartbeat.MessageModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;

/**
 * @Description: 服务启动时初始化运行，哪个微服务模块需要则拿此模版去用
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/3
 */
@Component
public class StartRunning implements CommandLineRunner {
    private final static Logger log = LoggerFactory.getLogger(StartRunning.class);

    @Resource
    RocketMessageConsumer rocketMessageConsumer;

    @Value("rocketmq.name-server")
    String nameServer;

    @Override
    public void run(String... args) throws Exception {
        log.info("服务器启动时执行 start");
        /**
         * session mq start
         */
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.sessionConsumerWebGroup.getCode(), MqTopicEnum.sessionTopic.getCode(), MqTagEnum.web.name(), MessageModel.CLUSTERING, new SessionConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.sessionConsumerWinGroup.getCode(), MqTopicEnum.sessionTopic.getCode(), MqTagEnum.win.name(), MessageModel.CLUSTERING, new SessionConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.sessionConsumerMacGroup.getCode(), MqTopicEnum.sessionTopic.getCode(), MqTagEnum.mac.name(), MessageModel.CLUSTERING, new SessionConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.sessionConsumerWxappGroup.getCode(), MqTopicEnum.sessionTopic.getCode(), MqTagEnum.wxapp.name(), MessageModel.CLUSTERING, new SessionConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.sessionConsumerIosGroup.getCode(), MqTopicEnum.sessionTopic.getCode(), MqTagEnum.ios.name(), MessageModel.CLUSTERING, new SessionConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.sessionConsumerAndroidGroup.getCode(), MqTopicEnum.sessionTopic.getCode(), MqTagEnum.android.name(), MessageModel.CLUSTERING, new SessionConcurrentlyImpl());
        /**
         * session mq end
         */

        /**
         * userLog mq start
         */
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.userLogConsumerUserGroup.getCode(), MqTopicEnum.userLogTopic.getCode(), MqTagEnum.user.name(), MessageModel.CLUSTERING, new UserLogConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.userLogConsumerStudentGroup.getCode(), MqTopicEnum.userLogTopic.getCode(), MqTagEnum.student.name(), MessageModel.CLUSTERING, new UserLogConcurrentlyImpl());
        /**
         * userLog mq end
         */

        /**
         * task mq start
         */
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.taskConsumerExamStudentImportGroup.getCode(), MqTopicEnum.taskTopic.getCode(), MqTagEnum.examStudentImport.name(), MessageModel.CLUSTERING, new TaskConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.taskConsumerRoomCodeImportGroup.getCode(), MqTopicEnum.taskTopic.getCode(), MqTagEnum.roomCodeImport.name(), MessageModel.CLUSTERING, new TaskConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.taskConsumerRoomCodeExportGroup.getCode(), MqTopicEnum.taskTopic.getCode(), MqTagEnum.roomCodeExport.name(), MessageModel.CLUSTERING, new TaskConcurrentlyImpl());
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.taskConsumerExamPaperImportGroup.getCode(), MqTopicEnum.taskTopic.getCode(), MqTagEnum.examPaperImport.name(), MessageModel.CLUSTERING, new TaskConcurrentlyImpl());
        /**
         * task mq end
         */

        /**
         * websocket mq start
         */
        rocketMessageConsumer.setRocketMQConsumer(nameServer, MqGroupEnum.websocketUnNormalConsumerOeGroup.getCode(), MqTopicEnum.websocketUnNormalTopic.getCode(), MqTagEnum.oe.name(), MessageModel.CLUSTERING, new WebsocketUnNormalConcurrentlyImpl());
        /**
         * websocket mq end
         */
        SystemConstant.initTempFiles();
        log.info("服务器启动时执行 end");
    }
}
