package com.qmth.themis.business.service.impl;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.themis.business.cache.bean.ExamPaperCacheBean;
import com.qmth.themis.business.cache.bean.ObjectiveAnswerCacheBean;
import com.qmth.themis.business.config.SystemConfig;
import com.qmth.themis.business.dao.TEExamPaperMapper;
import com.qmth.themis.business.entity.TEExamPaper;
import com.qmth.themis.business.service.TEExamPaperService;
import com.qmth.themis.business.util.OssUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.FileUtil;

/**
 * @Description: 考试试卷 服务实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Service
public class TEExamPaperServiceImpl extends ServiceImpl<TEExamPaperMapper, TEExamPaper> implements TEExamPaperService {
    @Resource
    SystemConfig systemConfig;
	@Resource
	TEExamPaperMapper teExamPaperMapper;

	@Override
	public TEExamPaper findByExamIdAndCourseCodeAndPaperCode(Long examId, String courseCode, String paperCode) {
		if (examId == null) {
			throw new BusinessException("examId is null");
		}
		if (StringUtils.isBlank(courseCode)) {
			throw new BusinessException("courseCode is null");
		}
		if (StringUtils.isBlank(paperCode)) {
			throw new BusinessException("paperCode is null");
		}
		return teExamPaperMapper.findByExamIdAndCourseCodeAndPaperCode(examId, courseCode, paperCode);
	}
	
	@Cacheable(value = "exam_paper", key = "#paperId", unless = "#result == null")
	@Override
	public ExamPaperCacheBean getExamPaperCacheBean(Long paperId) {
		ExamPaperCacheBean ret = null;
		TEExamPaper ep = getById(paperId);
		if (ep == null) {
			return ret;
		}
		ret = new ExamPaperCacheBean();
		ret.setId(ep.getId());
		ret.setExamId(ep.getExamId());
		ret.setCourseCode(ep.getCourseCode());
		ret.setCode(ep.getCode());
		ret.setName(ep.getName());
		ret.setTotalScore(ep.getTotalScore());
		ret.setPaperPath(ep.getPaperPath());
		ret.setStructPath(ep.getStructPath());
		ret.setAnswerPath(ep.getAnswerPath());
		ret.setDecryptSecret(ep.getDecryptSecret());
		ret.setDecryptVector(ep.getDecryptVector());
		ret.setEncryptMode(ep.getEncryptMode());
		ret.setNeedVoiceAnswer(ep.getNeedVoiceAnswer());
		ret.setHasAudio(ep.getHasAudio());
		ret.setWeight(ep.getWeight());
		ret.setAudioPlayCount(ep.getAudioPlayCount());
		return ret;
	}
	
	@Cacheable(value = "objective_answer", key = "#paperId", unless = "#result == null")
	@Override
	public Map<String,ObjectiveAnswerCacheBean> getObjectiveAnswerCacheBean(Long paperId) {
		Map<String,ObjectiveAnswerCacheBean> ret = null;
		ExamPaperCacheBean ep = getExamPaperCacheBean(paperId);
		if (ep == null) {
			return ret;
		}
		ret = new HashMap<String,ObjectiveAnswerCacheBean>();
		String structUrl=OssUtil.getUrlForPrivateBucket(systemConfig.getOssEnv(3), ep.getStructPath());
		String answerUrl=OssUtil.getUrlForPrivateBucket(systemConfig.getOssEnv(3), ep.getAnswerPath());
		String tempDir = systemConfig.getProperty("sys.config.tempDataDir");
        String dir = tempDir + "/" + uuid() + "/";
        File dfile = new File(dir);
        try {
            dfile.mkdirs();
            File structFile=new File(dir+"struct.json");
            FileUtil.saveUrlAsFile(structUrl, structFile);
            File answerEncryptFile=new File(dir+"answer_encrypt_file.json");
            FileUtil.saveUrlAsFile(answerUrl, answerEncryptFile);
            File answerFile=new File(dir+"answer_file.json");
            FileUtil.decryptFile(answerEncryptFile, answerFile, ep.getDecryptSecret(), ep.getDecryptVector());
            String structJson=FileUtil.readFileContent(structFile);
            String answerjson=FileUtil.readFileContent(answerFile);
        } finally {
            FileUtil.deleteFolder(dir);
        }
		return ret;
	}
    private String uuid() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }
}
