package com.qmth.themis.mq.templete.impl;

import com.google.gson.Gson;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TMRocketMessage;
import com.qmth.themis.business.enums.MqEnum;
import com.qmth.themis.business.enums.SystemOperationEnum;
import com.qmth.themis.business.service.TBSessionService;
import com.qmth.themis.business.service.TEExamStudentLogService;
import com.qmth.themis.business.service.TEUserLogService;
import com.qmth.themis.business.service.TMRocketMessageService;
import com.qmth.themis.business.templete.TaskExportTemplete;
import com.qmth.themis.business.templete.TaskImportTemplete;
import com.qmth.themis.business.templete.impl.TaskExamStudentImportTemplete;
import com.qmth.themis.business.templete.impl.TaskRoomCodeExportTemplete;
import com.qmth.themis.business.templete.impl.TaskRoomCodeImportTemplete;
import com.qmth.themis.business.threadPool.MyThreadPool;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.mq.dto.MqDto;
import com.qmth.themis.mq.templete.Concurrently;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.common.message.MessageExt;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: mq 任务并行消费监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/28
 */
@Service
public class TaskConcurrentlyImpl implements Concurrently {
    private final static Logger log = LoggerFactory.getLogger(TaskConcurrentlyImpl.class);

    @Override
    @Transactional
    public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext consumeConcurrentlyContext) {
        RedisUtil redisUtil = SpringContextHolder.getBean(RedisUtil.class);
        MyThreadPool myThreadPool = SpringContextHolder.getBean(MyThreadPool.class);
        TMRocketMessageService tmRocketMessageService = SpringContextHolder.getBean(TMRocketMessageService.class);
        MqDto mqDto = null;
        try {
            long threadId = Thread.currentThread().getId();
            String threadName = Thread.currentThread().getName();
            Gson gson = new Gson();
            for (MessageExt messageExt : msgs) {
                log.info(":{}-:{} task Consumer重试次数:{}", threadId, threadName, messageExt.getReconsumeTimes());
                mqDto = JacksonUtil.readJson(new String(messageExt.getBody(), Constants.CHARSET), MqDto.class);
                log.info(":{}-:{} task Consumer接收到的消息:{}", threadId, threadName, JacksonUtil.parseJson(mqDto));
                int reconsumeTime = messageExt.getReconsumeTimes();
                if (reconsumeTime >= SystemConstant.MAXRECONSUMETIMES) {
                    //超过最大重试次数，保存到数据库，后续可以发短信通知系统管理人员
                    mqDto.setAck(SystemConstant.POSION_ACK_TYPE);
                    TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                    tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                    redisUtil.delete(SystemConstant.MQ_TOPIC_BUFFER_LIST, mqDto.getId());
                } else {
                    if (Objects.nonNull(mqDto.getAck()) && mqDto.getAck().intValue() != SystemConstant.STANDARD_ACK_TYPE && Objects.nonNull(redisUtil.get(SystemConstant.MQ_TOPIC_BUFFER_LIST, mqDto.getId())) && redisUtil.lock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId(), SystemConstant.REDIS_LOCK_MQ_TIME_OUT)) {
                        Map<String, Object> map = (Map<String, Object>) mqDto.getBody();
                        String tag = mqDto.getTag();
                        myThreadPool.arbitratePoolTaskExecutor.execute(() -> {
                            if (tag.contains("Import")) {
                                TaskImportTemplete taskImportTemplete = null;
                                if (tag.contains("examStudentImport")) {
                                    taskImportTemplete = new TaskExamStudentImportTemplete();
                                } else if (tag.contains("roomCodeImport")) {
                                    taskImportTemplete = new TaskRoomCodeImportTemplete();
                                } else if (tag.contains("examPaperImport")) {
                                    taskImportTemplete = SpringContextHolder.getBean("taskExamPaperImportTemplete");
                                }
                                try {
                                    taskImportTemplete.importTask(map);
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }
                            } else {
                                TaskExportTemplete taskExportTemplete = null;
                                if (tag.contains("roomCodeExport")) {
                                    taskExportTemplete = new TaskRoomCodeExportTemplete();
                                }
                                try {
                                    taskExportTemplete.exportTask(map);
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }
                            }
                        });
                        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
                        mqDto.setBody(JacksonUtil.parseJson(mqDto.getBody()));
                        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                        redisUtil.delete(SystemConstant.MQ_TOPIC_BUFFER_LIST + mqDto.getId());
                        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                    } else {
                        log.info(":{}-:{} 消息ack未确认，重发", threadId, threadName);
                        return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
        } finally {
            if (Objects.nonNull(mqDto)) {
                redisUtil.releaseLock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId());
            }
        }
        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
    }
}
