package com.qmth.themis.mq.templete.impl;

import com.google.gson.Gson;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TMRocketMessage;
import com.qmth.themis.business.enums.MqEnum;
import com.qmth.themis.business.enums.SystemOperationEnum;
import com.qmth.themis.business.service.TEExamStudentLogService;
import com.qmth.themis.business.service.TEUserLogService;
import com.qmth.themis.business.service.TMRocketMessageService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.mq.dto.MqDto;
import com.qmth.themis.mq.templete.Concurrently;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.common.message.MessageExt;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * @Description: mq 用户轨迹并行消费监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/28
 */
@Service
public class UserLogConcurrentlyImpl implements Concurrently {
    private final static Logger log = LoggerFactory.getLogger(UserLogConcurrentlyImpl.class);

    @Override
    @Transactional
    public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext consumeConcurrentlyContext) {
        RedisUtil redisUtil = SpringContextHolder.getBean(RedisUtil.class);
        TEUserLogService teUserLogService = SpringContextHolder.getBean(TEUserLogService.class);
        TEExamStudentLogService teExamStudentLogService = SpringContextHolder.getBean(TEExamStudentLogService.class);
        TMRocketMessageService tmRocketMessageService = SpringContextHolder.getBean(TMRocketMessageService.class);
        MqDto mqDto = null;
        try {
            long threadId = Thread.currentThread().getId();
            String threadName = Thread.currentThread().getName();
            Gson gson = new Gson();
            for (MessageExt messageExt : msgs) {
                mqDto = JacksonUtil.readJson(new String(messageExt.getBody(), Constants.CHARSET), MqDto.class);
                log.info(":{}-:{} logConsumer重试次数:{}", threadId, threadName, messageExt.getReconsumeTimes());
//                MqDto mqDto = (MqDto) toJavaObject(parseObject(new String(messageExt.getBody(), Constants.CHARSET)), MqDto.class);
                log.info(":{}-:{} logConsumer接收到的消息:{}", threadId, threadName, JacksonUtil.parseJson(mqDto));
                int reconsumeTime = messageExt.getReconsumeTimes();
                if (reconsumeTime >= SystemConstant.MAXRECONSUMETIMES) {
                    //超过最大重试次数，保存到数据库，后续可以发短信通知系统管理人员
                    mqDto.setAck(SystemConstant.POSION_ACK_TYPE);
                    TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                    tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                    redisUtil.delete(SystemConstant.MQ_TOPIC_BUFFER_LIST, mqDto.getId());
                } else {
                    if (Objects.nonNull(mqDto.getAck()) && mqDto.getAck().intValue() != SystemConstant.STANDARD_ACK_TYPE && Objects.nonNull(redisUtil.get(SystemConstant.MQ_TOPIC_BUFFER_LIST, mqDto.getId())) && redisUtil.lock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId(), SystemConstant.REDIS_LOCK_MQ_TIME_OUT)) {
                        log.info(":{}-:{} 插入用户轨迹日志", threadId, threadName);
                        String tag = mqDto.getTag();
                        if (tag.contains("user")) {
                            teUserLogService.saveUserLogInfo(mqDto.getTimestamp(), mqDto.getObjId(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
                        } else if (tag.contains("student")) {
                            teExamStudentLogService.saveStudentLogInfo(mqDto.getTimestamp(), MqEnum.valueOf(String.valueOf(mqDto.getType())).getId(), SystemOperationEnum.valueOf(String.valueOf(mqDto.getBody())).getCode(), JacksonUtil.parseJson(mqDto));
                        }
                        mqDto.setAck(SystemConstant.STANDARD_ACK_TYPE);
                        TMRocketMessage tmRocketMessage = gson.fromJson(gson.toJson(mqDto), TMRocketMessage.class);
                        tmRocketMessageService.saveOrUpdate(tmRocketMessage);
                        redisUtil.delete(SystemConstant.MQ_TOPIC_BUFFER_LIST, mqDto.getId());
                        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                    } else {
                        log.info(":{}-:{} 消息ack未确认，重发", threadId, threadName);
                        return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
        } finally {
            if (Objects.nonNull(mqDto)) {
                redisUtil.releaseLock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId());
            }
        }
        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
    }
}
